@@ -27,28 +27,18 @@
 #include <kcmdlineargs.h>
 #include <kglobal.h>
 
-#include <QFile>
 
 extern "C" {
 	int KDE_EXPORT kdemain( int argc, char **argv )
 	{
-		// KApplication is necessary to use other ioslaves
-		putenv(strdup("SESSION_MANAGER="));
-		KCmdLineArgs::init(argc, argv, "kio_remote", 0, ki18n(0L), false, ki18n(0L));
+        // necessary to use other kio slaves
+        KComponentData componentData("kio_remote" );
+        QCoreApplication app(argc, argv);
 
-		KCmdLineOptions options;
-		options.add("+protocol", ki18n( "Protocol name" ));
-		options.add("+pool", ki18n( "Socket name" ));
-		options.add("+app", ki18n( "Socket name" ));
-		KCmdLineArgs::addCmdLineOptions( options );
-		KApplication app(  false );
-
-		KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
-		RemoteProtocol slave( QFile::encodeName( args->arg(0) ), 
-		                      QFile::encodeName( args->arg(1) ),
-		                      QFile::encodeName( args->arg(2) ) );
-		slave.dispatchLoop();
-		return 0;
+        // start the slave
+        RemoteProtocol slave( argv[1], argv[2], argv[3] );
+        slave.dispatchLoop();
+        return 0;
 	}
 }
 
@@ -74,13 +64,17 @@ void RemoteProtocol::listDir(const KUrl &url)
 	}
 
 	int second_slash_idx = url.path().indexOf( '/', 1 );
-	QString root_dirname = url.path().mid( 1, second_slash_idx-1 );
-	
+	const QString root_dirname = url.path().mid( 1, second_slash_idx-1 );
+
 	KUrl target = m_impl.findBaseURL( root_dirname );
 	kDebug(1220) << "possible redirection target : " << target;
 	if( target.isValid() )
 	{
+		if ( second_slash_idx < 0 ) {
+			second_slash_idx = url.path().size();
+		}
 		target.addPath( url.path().remove(0, second_slash_idx) );
+		kDebug(1220) << "complete redirection target : " << target;
 		redirection(target);
 		finished();
 		return;
@@ -104,8 +98,8 @@ void RemoteProtocol::listRoot()
 	m_impl.createWizardEntry(entry);
 	listEntry(entry, false);
 
-	KIO::UDSEntryList::ConstIterator it = remote_entries.begin();
-	const KIO::UDSEntryList::ConstIterator end = remote_entries.end();
+	KIO::UDSEntryList::ConstIterator it = remote_entries.constBegin();
+	const KIO::UDSEntryList::ConstIterator end = remote_entries.constEnd();
 	for(; it!=end; ++it)
 	{
 		listEntry(*it, false);
@@ -148,8 +142,8 @@ void RemoteProtocol::stat(const KUrl &url)
 	}
 
 	int second_slash_idx = url.path().indexOf( '/', 1 );
-	QString root_dirname = url.path().mid( 1, second_slash_idx-1 );
-	
+	const QString root_dirname = url.path().mid( 1, second_slash_idx-1 );
+
 	if ( second_slash_idx==-1 || ( (int)url.path().length() )==second_slash_idx+1 )
 	{
 		KIO::UDSEntry entry;
@@ -166,13 +160,17 @@ void RemoteProtocol::stat(const KUrl &url)
 		kDebug( 1220 ) << "possible redirection target : " << target;
 		if (  target.isValid() )
 		{
+			if ( second_slash_idx < 0 ) {
+				second_slash_idx = url.path().size();
+			}
+			kDebug(1220) << "complete redirection target : " << target;
 			target.addPath( url.path().remove( 0, second_slash_idx ) );
 			redirection( target );
 			finished();
 			return;
 		}
 	}
-	
+
 	error(KIO::ERR_MALFORMED_URL, url.prettyUrl());
 }
 
@@ -194,7 +192,7 @@ void RemoteProtocol::get(const KUrl &url)
 {
 	kDebug(1220) << "RemoteProtocol::get: " << url;
 
-	QString file = m_impl.findDesktopFile( url.fileName() );
+	const QString file = m_impl.findDesktopFile( url.fileName() );
 	kDebug(1220) << "desktop file : " << file;
 
 	if (!file.isEmpty())
@@ -228,3 +226,14 @@ void RemoteProtocol::rename(const KUrl &src, const KUrl &dest,
 
 	error(KIO::ERR_CANNOT_RENAME, src.prettyUrl());
 }
+
+void RemoteProtocol::symlink(const QString &target, const KUrl &dest, KIO::JobFlags flags)
+{
+	if (m_impl.changeFolderTarget(dest.fileName(), target, flags & KIO::Overwrite))
+	{
+		finished();
+		return;
+	}
+
+	error(KIO::ERR_CANNOT_SYMLINK, dest.prettyUrl());
+}
