@@ -17,7 +17,12 @@
    Boston, MA 02110-1301, USA.
 */
 
+#define QT_NO_CAST_FROM_ASCII
+
 #include "trashimpl.h"
+#include "discspaceutil.h"
+#include "trashsizecache.h"
+
 #include <klocale.h>
 #include <kde_file.h>
 #include <kio/global.h>
@@ -36,7 +41,6 @@
 #include <kconfiggroup.h>
 #include <kmountpoint.h>
 
-#include <QApplication>
 #include <QEventLoop>
 #include <QFile>
 #include <QDir>
@@ -63,7 +67,7 @@ TrashImpl::TrashImpl() :
     m_trashDirectoriesScanned( false ),
     // not using kio_trashrc since KIO uses that one already for kio_trash
     // so better have a separate one, for faster parsing by e.g. kmimetype.cpp
-    m_config( "trashrc" )
+    m_config(QString::fromLatin1("trashrc"), KConfig::SimpleConfig)
 {
     KDE_struct_stat buff;
     if ( KDE_lstat( QFile::encodeName( QDir::homePath() ), &buff ) == 0 ) {
@@ -85,7 +89,7 @@ int TrashImpl::testDir( const QString &_name ) const
   if ( dp == NULL )
   {
     QString name = _name;
-    if ( name.endsWith( '/' ) )
+    if (name.endsWith(QLatin1Char('/')))
       name.truncate( name.length() - 1 );
     QByteArray path = QFile::encodeName(name);
 
@@ -95,7 +99,7 @@ int TrashImpl::testDir( const QString &_name ) const
         //int ret = KMessageBox::warningYesNo( 0, i18n("%1 is a file, but KDE needs it to be a directory. Move it to %2.orig and create directory?").arg(name).arg(name) );
         //if ( ret == KMessageBox::Yes ) {
 #endif
-            if ( ::rename( path, path + ".orig" ) == 0 ) {
+            if ( KDE_rename( path, path + ".orig" ) == 0 ) {
                 ok = KDE_mkdir( path, S_IRWXU ) == 0;
             } else { // foo.orig existed already. How likely is that?
                 ok = false;
@@ -112,10 +116,10 @@ int TrashImpl::testDir( const QString &_name ) const
     if ( !ok )
     {
         //KMessageBox::sorry( 0, i18n( "Could not create directory %1. Check for permissions." ).arg( name ) );
-        kWarning() << "could not create " << name ;
+        kWarning() << "could not create" << name ;
         return KIO::ERR_COULD_NOT_MKDIR;
     } else {
-        kDebug() << name << " created.";
+        kDebug() << name << "created.";
     }
   }
   else // exists already
@@ -142,18 +146,18 @@ bool TrashImpl::init()
         return false;
     }
 
-    const QString trashDir = xdgDataDir + "Trash";
+    const QString trashDir = xdgDataDir + QString::fromLatin1("Trash");
     int err;
     if ( ( err = testDir( trashDir ) ) ) {
         error( err, trashDir );
         return false;
     }
-    if ( ( err = testDir( trashDir + "/info" ) ) ) {
-        error( err, trashDir + "/info" );
+    if ( ( err = testDir( trashDir + QString::fromLatin1("/info") ) ) ) {
+        error( err, trashDir + QString::fromLatin1("/info") );
         return false;
     }
-    if ( ( err = testDir( trashDir + "/files" ) ) ) {
-        error( err, trashDir + "/files" );
+    if ( ( err = testDir( trashDir + QString::fromLatin1("/files") ) ) ) {
+        error( err, trashDir + QString::fromLatin1("/files") );
         return false;
     }
     m_trashDirectories.insert( 0, trashDir );
@@ -178,7 +182,7 @@ void TrashImpl::migrateOldTrash()
           entryIt != entryEnd ; ++entryIt )
     {
         QString srcPath = *entryIt;
-        if ( srcPath == "." || srcPath == ".." || srcPath == ".directory" )
+        if ( srcPath == QLatin1String(".") || srcPath == QLatin1String("..") || srcPath == QLatin1String(".directory") )
             continue;
         srcPath.prepend( oldTrashDir ); // make absolute
         int trashId;
@@ -209,6 +213,13 @@ bool TrashImpl::createInfo( const QString& origPath, int& trashId, QString& file
     kDebug() << origPath;
     // Check source
     const QByteArray origPath_c( QFile::encodeName( origPath ) );
+    /*
+         * off_t should be 64bit on Unix systems to have large file support
+         * FIXME: on windows this gets disabled until trash gets integrated
+         */
+#ifndef Q_OS_WIN
+    char off_t_should_be_64bit[sizeof(off_t) >= 8 ? 1:-1]; (void)off_t_should_be_64bit;
+#endif
     KDE_struct_stat buff_src;
     if ( KDE_lstat( origPath_c.data(), &buff_src ) == -1 ) {
         if ( errno == EACCES )
@@ -239,7 +250,7 @@ bool TrashImpl::createInfo( const QString& origPath, int& trashId, QString& file
     int fd = 0;
     do {
         kDebug() << "trying to create " << url.path() ;
-        fd = ::open( QFile::encodeName( url.path() ), O_WRONLY | O_CREAT | O_EXCL, 0600 );
+        fd = KDE_open( QFile::encodeName( url.path() ), O_WRONLY | O_CREAT | O_EXCL, 0600 );
         if ( fd < 0 ) {
             if ( errno == EEXIST ) {
                 url.setFileName( KIO::RenameDialog::suggestName( baseDirectory, url.fileName() ) );
@@ -252,7 +263,7 @@ bool TrashImpl::createInfo( const QString& origPath, int& trashId, QString& file
     } while ( fd < 0 );
     const QString infoPath = url.path();
     fileId = url.fileName();
-    Q_ASSERT( fileId.endsWith( ".trashinfo" ) );
+    Q_ASSERT( fileId.endsWith( QLatin1String(".trashinfo") ) );
     fileId.truncate( fileId.length() - 10 ); // remove .trashinfo from fileId
 
     FILE* file = ::fdopen( fd, "w" );
@@ -297,7 +308,7 @@ QString TrashImpl::makeRelativePath( const QString& topdir, const QString& path
     // topdir ends with '/'
     if ( realPath.startsWith( topdir ) ) {
         const QString rel = realPath.mid( topdir.length() );
-        Q_ASSERT( rel[0] != '/' );
+        Q_ASSERT( rel[0] != QLatin1Char('/') );
         return rel;
     } else { // shouldn't happen...
         kWarning() << "Couldn't make relative path for " << realPath << " (" << path << "), with topdir=" << topdir ;
@@ -316,16 +327,16 @@ void TrashImpl::enterLoop()
 QString TrashImpl::infoPath( int trashId, const QString& fileId ) const
 {
     QString trashPath = trashDirectoryPath( trashId );
-    trashPath += "/info/";
+    trashPath += QString::fromLatin1("/info/");
     trashPath += fileId;
-    trashPath += ".trashinfo";
+    trashPath += QString::fromLatin1(".trashinfo");
     return trashPath;
 }
 
 QString TrashImpl::filesPath( int trashId, const QString& fileId ) const
 {
     QString trashPath = trashDirectoryPath( trashId );
-    trashPath += "/files/";
+    trashPath += QString::fromLatin1("/files/");
     trashPath += fileId;
     return trashPath;
 }
@@ -341,6 +352,14 @@ bool TrashImpl::deleteInfo( int trashId, const QString& fileId )
 bool TrashImpl::moveToTrash( const QString& origPath, int trashId, const QString& fileId )
 {
     kDebug() ;
+    if ( !adaptTrashSize( origPath, trashId ) )
+        return false;
+
+    const qulonglong pathSize = DiscSpaceUtil::sizeOfPath( origPath );
+
+    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+    trashSize.initialize();
+
     const QString dest = filesPath( trashId, fileId );
     if ( !move( origPath, dest ) ) {
         // Maybe the move failed due to no permissions to delete source.
@@ -351,6 +370,9 @@ bool TrashImpl::moveToTrash( const QString& origPath, int trashId, const QString
             synchronousDel( dest, false, true );
         return false;
     }
+
+    trashSize.add( pathSize );
+
     fileAdded();
     return true;
 }
@@ -359,11 +381,19 @@ bool TrashImpl::moveFromTrash( const QString& dest, int trashId, const QString&
 {
     QString src = filesPath( trashId, fileId );
     if ( !relativePath.isEmpty() ) {
-        src += '/';
+        src += QLatin1Char('/');
         src += relativePath;
     }
+    const qulonglong pathSize = DiscSpaceUtil::sizeOfPath( src );
+
+    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+    trashSize.initialize();
+
     if ( !move( src, dest ) )
         return false;
+
+    trashSize.remove( pathSize );
+
     return true;
 }
 
@@ -401,9 +431,20 @@ void TrashImpl::jobFinished(KJob* job)
 bool TrashImpl::copyToTrash( const QString& origPath, int trashId, const QString& fileId )
 {
     kDebug() ;
+    if ( !adaptTrashSize( origPath, trashId ) )
+        return false;
+
+    const qulonglong pathSize = DiscSpaceUtil::sizeOfPath( origPath );
+
+    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+    trashSize.initialize();
+
     const QString dest = filesPath( trashId, fileId );
     if ( !copy( origPath, dest ) )
         return false;
+
+    trashSize.add( pathSize );
+
     fileAdded();
     return true;
 }
@@ -412,7 +453,7 @@ bool TrashImpl::copyFromTrash( const QString& dest, int trashId, const QString&
 {
     QString src = filesPath( trashId, fileId );
     if ( !relativePath.isEmpty() ) {
-        src += '/';
+        src += QLatin1Char('/');
         src += relativePath;
     }
     return copy( src, dest );
@@ -439,7 +480,7 @@ bool TrashImpl::copy( const QString& src, const QString& dest )
 bool TrashImpl::directRename( const QString& src, const QString& dest )
 {
     kDebug() << src << " -> " << dest;
-    if ( ::rename( QFile::encodeName( src ), QFile::encodeName( dest ) ) != 0 ) {
+    if ( KDE_rename( QFile::encodeName( src ), QFile::encodeName( dest ) ) != 0 ) {
         if (errno == EXDEV) {
             error( KIO::ERR_UNSUPPORTED_ACTION, QString::fromLatin1("rename") );
         } else {
@@ -484,6 +525,8 @@ bool TrashImpl::del( int trashId, const QString& fileId )
     QString info = infoPath(trashId, fileId);
     QString file = filesPath(trashId, fileId);
 
+    const qulonglong fileSize = DiscSpaceUtil::sizeOfPath( file );
+
     QByteArray info_c = QFile::encodeName(info);
 
     KDE_struct_stat buff;
@@ -495,9 +538,14 @@ bool TrashImpl::del( int trashId, const QString& fileId )
         return false;
     }
 
+    TrashSizeCache trashSize( trashDirectoryPath( trashId ) );
+    trashSize.initialize();
+
     if ( !synchronousDel( file, true, QFileInfo(file).isDir() ) )
         return false;
 
+    trashSize.remove( fileSize );
+
     QFile::remove( info );
     fileRemoved();
     return true;
@@ -513,7 +561,7 @@ bool TrashImpl::synchronousDel( const QString& path, bool setLastErrorCode, bool
     // otherwise we won't be able to delete files in them (#130780).
     if ( isDir ) {
         kDebug() << "chmod'ing " << url;
-        KFileItem fileItem( url, "inode/directory", KFileItem::Unknown );
+        KFileItem fileItem( url, QString::fromLatin1("inode/directory"), KFileItem::Unknown );
         KFileItemList fileItemList;
         fileItemList.append( fileItem );
         KIO::ChmodJob* chmodJob = KIO::chmod( fileItemList, 0200, 0200, QString(), QString(), true /*recursive*/, KIO::HideProgressInfo );
@@ -540,8 +588,15 @@ bool TrashImpl::emptyTrash()
     // The naive implementation "delete info and files in every trash directory"
     // breaks when deleted directories contain files owned by other users.
     // We need to ensure that the .trashinfo file is only removed when the
-    // corresponding files could indeed be removed.
+    // corresponding files could indeed be removed (#116371)
 
+    // On the other hand, we certainly want to remove any file that has no associated
+    // .trashinfo file for some reason (#167051)
+
+    QSet<QString> unremoveableFiles;
+
+    int myErrorCode = 0;
+    QString myErrorMsg;
     const TrashedFileInfoList fileInfoList = list();
 
     TrashedFileInfoList::const_iterator it = fileInfoList.begin();
@@ -549,10 +604,42 @@ bool TrashImpl::emptyTrash()
     for ( ; it != end ; ++it ) {
         const TrashedFileInfo& info = *it;
         const QString filesPath = info.physicalPath;
-        if ( synchronousDel( filesPath, true, true ) ) {
+        if ( synchronousDel( filesPath, true, true ) || m_lastErrorCode == KIO::ERR_DOES_NOT_EXIST) {
             QFile::remove( infoPath( info.trashId, info.fileId ) );
-        } // else error code is set
+        } else {
+            // error code is set by synchronousDel, let's remember it
+            // (so that successfully removing another file doesn't erase the error)
+            myErrorCode = m_lastErrorCode;
+            myErrorMsg = m_lastErrorMessage;
+            // and remember not to remove this file
+            unremoveableFiles.insert(filesPath);
+            kDebug() << "Unremoveable:" << filesPath;
+        }
+
+        TrashSizeCache trashSize( trashDirectoryPath( info.trashId ) );
+        trashSize.clear();
     }
+
+    // Now do the orphaned-files cleanup
+    TrashDirMap::const_iterator trit = m_trashDirectories.constBegin();
+    for (; trit != m_trashDirectories.constEnd() ; ++trit) {
+        //const int trashId = trit.key();
+        QString filesDir = trit.value();
+        filesDir += QString::fromLatin1("/files");
+        Q_FOREACH(const QString& fileName, listDir(filesDir)) {
+            if (fileName == QString::fromLatin1(".") || fileName == QString::fromLatin1(".."))
+                continue;
+            const QString filePath = filesDir + QLatin1Char('/') + fileName;
+            if (!unremoveableFiles.contains(filePath)) {
+                kWarning() << "Removing orphaned file" << filePath;
+                QFile::remove(filePath);
+            }
+        }
+    }
+
+    m_lastErrorCode = myErrorCode;
+    m_lastErrorMessage = myErrorMsg;
+
     fileRemoved();
 
     return m_lastErrorCode == 0;
@@ -566,24 +653,24 @@ TrashImpl::TrashedFileInfoList TrashImpl::list()
 
     TrashedFileInfoList lst;
     // For each known trash directory...
-    TrashDirMap::const_iterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
+    TrashDirMap::const_iterator it = m_trashDirectories.constBegin();
+    for ( ; it != m_trashDirectories.constEnd() ; ++it ) {
         const int trashId = it.key();
         QString infoPath = it.value();
-        infoPath += "/info";
+        infoPath += QString::fromLatin1("/info");
         // Code taken from kio_file
-        QStringList entryNames = listDir( infoPath );
+        const QStringList entryNames = listDir( infoPath );
         //char path_buffer[PATH_MAX];
         //getcwd(path_buffer, PATH_MAX - 1);
         //if ( chdir( infoPathEnc ) )
         //    continue;
-        for ( QStringList::const_iterator entryIt = entryNames.begin(), entryEnd = entryNames.end();
+        for ( QStringList::const_iterator entryIt = entryNames.constBegin(), entryEnd = entryNames.constEnd();
               entryIt != entryEnd ; ++entryIt )
         {
             QString fileName = *entryIt;
-            if ( fileName == "." || fileName == ".." )
+            if ( fileName == QLatin1String(".") || fileName == QLatin1String("..") )
                 continue;
-            if ( !fileName.endsWith( ".trashinfo" ) ) {
+            if ( !fileName.endsWith( QLatin1String(".trashinfo") ) ) {
                 kWarning() << "Invalid info file found in " << infoPath << " : " << fileName ;
                 continue;
             }
@@ -624,9 +711,9 @@ bool TrashImpl::readInfoFile( const QString& infoPath, TrashedFileInfo& info, in
     info.origPath = QUrl::fromPercentEncoding( group.readEntry( "Path" ).toLatin1() );
     if ( info.origPath.isEmpty() )
         return false; // path is mandatory...
-    if ( trashId == 0 )
-        Q_ASSERT( info.origPath[0] == '/' );
-    else {
+    if ( trashId == 0 ) {
+        Q_ASSERT( info.origPath[0] == QLatin1Char('/') );
+    } else {
         const QString topdir = topDirectoryPath( trashId ); // includes trailing slash
         info.origPath.prepend( topdir );
     }
@@ -641,7 +728,7 @@ QString TrashImpl::physicalPath( int trashId, const QString& fileId, const QStri
 {
     QString filePath = filesPath( trashId, fileId );
     if ( !relativePath.isEmpty() ) {
-        filePath += '/';
+        filePath += QLatin1Char('/');
         filePath += relativePath;
     }
     return filePath;
@@ -660,18 +747,18 @@ bool TrashImpl::isEmpty() const
     // For each known trash directory...
     if ( !m_trashDirectoriesScanned )
         scanTrashDirectories();
-    TrashDirMap::const_iterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
+    TrashDirMap::const_iterator it = m_trashDirectories.constBegin();
+    for ( ; it != m_trashDirectories.constEnd() ; ++it ) {
         QString infoPath = it.value();
-        infoPath += "/info";
+        infoPath += QString::fromLatin1("/info");
 
         DIR *dp = opendir( QFile::encodeName( infoPath ) );
         if ( dp )
         {
-            struct dirent *ep;
-            ep = readdir( dp );
-            ep = readdir( dp ); // ignore '.' and '..' dirent
-            ep = readdir( dp ); // look for third file
+            KDE_struct_dirent *ep;
+            ep = KDE_readdir( dp );
+            ep = KDE_readdir( dp ); // ignore '.' and '..' dirent
+            ep = KDE_readdir( dp ); // look for third file
             closedir( dp );
             if ( ep != 0 ) {
                 //kDebug() << ep->d_name << " in " << infoPath << " -> not empty";
@@ -684,6 +771,7 @@ bool TrashImpl::isEmpty() const
 
 void TrashImpl::fileAdded()
 {
+    m_config.reparseConfiguration();
     KConfigGroup group = m_config.group( "Status" );
     if ( group.readEntry( "Empty", true) == true ) {
         group.writeEntry( "Empty", false );
@@ -723,8 +811,10 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
         return 0;
 
     KMountPoint::Ptr mp = KMountPoint::currentMountPoints().findByPath( origPath );
-    if (!mp)
+    if (!mp) {
+        kDebug() << "KMountPoint found no mount point for" << origPath;
         return 0;
+    }
     QString mountPoint = mp->mountPoint();
     const QString trashDir = trashForMountPoint( mountPoint, true );
     kDebug() << "mountPoint=" << mountPoint << " trashDir=" << trashDir;
@@ -747,7 +837,10 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
     return m_lastId;
 #endif
 
-    const QList<Solid::Device> lst = Solid::Device::listFromQuery("StorageAccess.accessible == true AND StorageAccess.filePath == '"+mountPoint+"'");
+    const QString query = QString::fromLatin1("[StorageAccess.accessible == true AND StorageAccess.filePath == '")+mountPoint+QString::fromLatin1("']");
+    //kDebug() << "doing solid query:" << query;
+    const QList<Solid::Device> lst = Solid::Device::listFromQuery(query);
+    //kDebug() << "got" << lst.count() << "devices";
     if ( lst.isEmpty() ) // not a device. Maybe some tmpfs mount for instance.
         return 0; // use the home trash instead
     // Pretend we got exactly one...
@@ -756,9 +849,9 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
     // new trash dir found, register it
     id = idForDevice( device );
     m_trashDirectories.insert( id, trashDir );
-    kDebug() << "found " << trashDir << " gave it id " << id;
-    if ( !mountPoint.endsWith( '/' ) )
-        mountPoint += '/';
+    kDebug() << "found" << trashDir << "gave it id" << id;
+    if (!mountPoint.endsWith(QLatin1Char('/')))
+        mountPoint += QLatin1Char('/');
     m_topDirectories.insert( id, mountPoint );
 
     return idForTrashDirectory( trashDir );
@@ -766,7 +859,7 @@ int TrashImpl::findTrashDirectory( const QString& origPath )
 
 void TrashImpl::scanTrashDirectories() const
 {
-    const QList<Solid::Device> lst = Solid::Device::listFromQuery("StorageAccess.accessible == true");
+    const QList<Solid::Device> lst = Solid::Device::listFromQuery(QString::fromLatin1("StorageAccess.accessible == true"));
     for ( QList<Solid::Device>::ConstIterator it = lst.begin() ; it != lst.end() ; ++it ) {
         QString topdir = (*it).as<Solid::StorageAccess>()->filePath();
         QString trashDir = trashForMountPoint( topdir, false );
@@ -778,8 +871,8 @@ void TrashImpl::scanTrashDirectories() const
                 trashId = idForDevice( *it );
                 m_trashDirectories.insert( trashId, trashDir );
                 kDebug() << "found " << trashDir << " gave it id " << trashId;
-                if ( !topdir.endsWith( '/' ) )
-                    topdir += '/';
+                if (!topdir.endsWith(QLatin1Char('/')))
+                    topdir += QLatin1Char('/');
                 m_topDirectories.insert( trashId, topdir );
             }
         }
@@ -805,7 +898,7 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
 {
     // (1) Administrator-created $topdir/.Trash directory
 
-    const QString rootTrashDir = topdir + "/.Trash";
+    const QString rootTrashDir = topdir + QString::fromLatin1("/.Trash");
     const QByteArray rootTrashDir_c = QFile::encodeName( rootTrashDir );
     // Can't use QFileInfo here since we need to test for the sticky bit
     uid_t uid = getuid();
@@ -815,9 +908,9 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
         if ( (S_ISDIR(buff.st_mode)) // must be a dir
              && (!S_ISLNK(buff.st_mode)) // not a symlink
              && ((buff.st_mode & requiredBits) == requiredBits)
-             && (::access(rootTrashDir_c, W_OK))
+             && (::access(rootTrashDir_c, W_OK) == 0) // must be user-writable
             ) {
-            const QString trashDir = rootTrashDir + '/' + QString::number( uid );
+            const QString trashDir = rootTrashDir + QLatin1Char('/') + QString::number( uid );
             const QByteArray trashDir_c = QFile::encodeName( trashDir );
             if ( KDE_lstat( trashDir_c, &buff ) == 0 ) {
                 if ( (buff.st_uid == uid) // must be owned by user
@@ -837,7 +930,7 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
     }
 
     // (2) $topdir/.Trash-$uid
-    const QString trashDir = topdir + "/.Trash-" + QString::number( uid );
+    const QString trashDir = topdir + QString::fromLatin1("/.Trash-") + QString::number( uid );
     const QByteArray trashDir_c = QFile::encodeName( trashDir );
     if ( KDE_lstat( trashDir_c, &buff ) == 0 )
     {
@@ -862,8 +955,8 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
 int TrashImpl::idForTrashDirectory( const QString& trashDir ) const
 {
     // If this is too slow we can always use a reverse map...
-    TrashDirMap::ConstIterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
+    TrashDirMap::ConstIterator it = m_trashDirectories.constBegin();
+    for ( ; it != m_trashDirectories.constEnd() ; ++it ) {
         if ( it.value() == trashDir ) {
             return it.key();
         }
@@ -905,10 +998,10 @@ bool TrashImpl::checkTrashSubdirs( const QByteArray& trashDir_c ) const
 {
     // testDir currently works with a QString - ## optimize
     QString trashDir = QFile::decodeName( trashDir_c );
-    const QString info = trashDir + "/info";
+    const QString info = trashDir + QString::fromLatin1("/info");
     if ( testDir( info ) != 0 )
         return false;
-    const QString files = trashDir + "/files";
+    const QString files = trashDir + QString::fromLatin1("/files");
     if ( testDir( files ) != 0 )
         return false;
     return true;
@@ -938,13 +1031,13 @@ QString TrashImpl::topDirectoryPath( int trashId ) const
 KUrl TrashImpl::makeURL( int trashId, const QString& fileId, const QString& relativePath )
 {
     KUrl url;
-    url.setProtocol( "trash" );
-    QString path = "/";
+    url.setProtocol(QString::fromLatin1("trash"));
+    QString path = QString::fromLatin1("/");
     path += QString::number( trashId );
-    path += '-';
+    path += QLatin1Char('-');
     path += fileId;
     if ( !relativePath.isEmpty() ) {
-        path += '/';
+        path += QLatin1Char('/');
         path += relativePath;
     }
     url.setPath( path );
@@ -955,13 +1048,13 @@ KUrl TrashImpl::makeURL( int trashId, const QString& fileId, const QString& rela
 // The trash:/ URL itself isn't parsed here, must be caught by the caller before hand.
 bool TrashImpl::parseURL( const KUrl& url, int& trashId, QString& fileId, QString& relativePath )
 {
-    if ( url.protocol() != "trash" )
+    if (url.protocol() != QLatin1String("trash"))
         return false;
     const QString path = url.path();
     int start = 0;
-    if ( path[0] == '/' ) // always true I hope
+    if ( path[0] == QLatin1Char('/') ) // always true I hope
         start = 1;
-    int slashPos = path.indexOf( '-', 0 ); // don't match leading slash
+    int slashPos = path.indexOf(QLatin1Char('-'), 0); // don't match leading slash
     if ( slashPos <= 0 )
         return false;
     bool ok = false;
@@ -970,7 +1063,7 @@ bool TrashImpl::parseURL( const KUrl& url, int& trashId, QString& fileId, QStrin
     if ( !ok )
         return false;
     start = slashPos + 1;
-    slashPos = path.indexOf( '/', start );
+    slashPos = path.indexOf(QLatin1Char('/'), start);
     if ( slashPos <= 0 ) {
         fileId = path.mid( start );
         relativePath.clear();
@@ -981,4 +1074,85 @@ bool TrashImpl::parseURL( const KUrl& url, int& trashId, QString& fileId, QStrin
     return true;
 }
 
+bool TrashImpl::adaptTrashSize( const QString& origPath, int trashId )
+{
+    KConfig config(QString::fromLatin1("ktrashrc"));
+
+    const QString trashPath = trashDirectoryPath( trashId );
+    KConfigGroup group = config.group( trashPath );
+
+    bool useTimeLimit = group.readEntry( "UseTimeLimit", false );
+    bool useSizeLimit = group.readEntry( "UseSizeLimit", true );
+    double percent = group.readEntry( "Percent", 10.0 );
+    int actionType = group.readEntry( "LimitReachedAction", 0 );
+
+    if ( useTimeLimit ) { // delete all files in trash older than X days
+        const int maxDays = group.readEntry( "Days", 7 );
+        const QDateTime currentDate = QDateTime::currentDateTime();
+
+        const TrashedFileInfoList trashedFiles = list();
+        for ( int i = 0; i < trashedFiles.count(); ++i ) {
+            struct TrashedFileInfo info = trashedFiles.at( i );
+            if ( info.trashId != trashId )
+                continue;
+
+            if ( info.deletionDate.daysTo( currentDate ) > maxDays )
+              del( info.trashId, info.fileId );
+        }
+    }
+
+    if ( useSizeLimit ) { // check if size limit exceeded
+
+        // calculate size of the files to be put into the trash
+        qulonglong additionalSize = DiscSpaceUtil::sizeOfPath( origPath );
+
+        const TrashSizeCache trashSize( trashPath );
+        DiscSpaceUtil util(trashPath + QString::fromLatin1("/files/"));
+        if ( util.usage( trashSize.size() + additionalSize ) >= percent ) {
+            if ( actionType == 0 ) { // warn the user only
+                m_lastErrorCode = KIO::ERR_SLAVE_DEFINED;
+                m_lastErrorMessage = i18n( "The trash has reached its maximum size!\nCleanup the trash manually." );
+                return false;
+            } else {
+                // before we start to remove any files from the trash,
+                // check whether the new file will fit into the trash
+                // at all...
+
+                qulonglong partitionSize = util.size();
+
+                if ( (((double)additionalSize/(double)partitionSize)*100) >= percent ) {
+                    m_lastErrorCode = KIO::ERR_SLAVE_DEFINED;
+                    m_lastErrorMessage = i18n( "The file is too large to be trashed." );
+                    return false;
+                }
+
+                // the size of the to be trashed file is ok, so lets start removing
+                // some other files from the trash
+
+                QDir dir(trashPath + QString::fromLatin1("/files"));
+                QFileInfoList infoList;
+                if ( actionType == 1 )  // delete oldest files first
+                    infoList = dir.entryInfoList( QDir::Files | QDir::AllDirs | QDir::NoDotAndDotDot, QDir::Time | QDir::Reversed );
+                else if ( actionType == 2 ) // delete biggest files first
+                    infoList = dir.entryInfoList( QDir::Files | QDir::AllDirs | QDir::NoDotAndDotDot, QDir::Size );
+                else
+                    qWarning( "Should never happen!" );
+
+                bool deleteFurther = true;
+                for ( int i = 0; (i < infoList.count()) && deleteFurther; ++i ) {
+                    const QFileInfo info = infoList.at( i );
+
+                    del( trashId, info.fileName() ); // delete trashed file
+
+                    const TrashSizeCache trashSize( trashPath );
+                    if ( util.usage( trashSize.size() + additionalSize ) < percent ) // check whether we have enough space now
+                         deleteFurther = false;
+                }
+            }
+        }
+    }
+
+    return true;
+}
+
 #include "trashimpl.moc"
