@@ -38,7 +38,7 @@ NotifyByTaskbar::~NotifyByTaskbar()
 
 void NotifyByTaskbar::notify( int id, KNotifyConfig * config )
 {
-	kDebug(300) ;
+	kDebug(300) << id << config->winId ;
 	
 	WId win = config->winId;
 	if( win != 0 )
